

%MACRO BRR_EFFECT(INFILE=,
				  REPLI_ROOT=,	
				  BYVAR=,
				  VAR=,
				  EFFECT=,
				  OUTFILE=);


/* 

MEANING OF THE MACRO ARGUMENTS

INFILE		= 	INPUT DATA FILE.
REPLI_ROOT	= 	ROOT OF THE FINAL WEIGHT AND 80 REPLICATES VARIABLE NAMES. 
				FINAL WEIGHT VARIABLE NAME MUST BE THE REPLICATION ROOT FOLLOWED BY 0.
BYVAR		= 	BREAKDOWN VARIABLES
VAR			= 	DEPENDENT VARIABLES
EFFECT		=	THIS ARGUMENT HAS THREE COMPONENTS: THE VARIABLE NAME AND THE TWO CATEGORIES ON WHICH THE EFFECT
				SIZE WILL BE COMPUTED. THE EFFECT SIZE WILL BE THE SECOND COMPONENT MINUS THE THIRD COMPONENT
OUTFILE		= 	FILE WITH THE STATISTIC ESTIMATES AND THEIR STANDARD ERROR ESTIMATE

*/ 



%LET GROUP=%SCAN(&EFFECT,1);
%LET CAT1=%SCAN(&EFFECT,2);
%LET CAT2=%SCAN(&EFFECT,3);


OPTIONS NONOTES;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP1;
RUN;

DATA BRRDATA;
	SET &INFILE;
	IF (&GROUP=&CAT1) THEN VAR_GROUP=1;
	IF (&GROUP=&CAT2) THEN VAR_GROUP=2;
	IF (VAR_GROUP IN (1,2));
	KEEP &REPLI_ROOT.0-&REPLI_ROOT.80 &BYVAR &VAR VAR_GROUP;
RUN;

PROC SORT DATA=BRRDATA;
  BY &BYVAR VAR_GROUP;
RUN;

%DO I = 0 %TO 80;
 	PROC MEANS DATA=BRRDATA VARDEF=WGT NOPRINT;
		VAR &VAR ;
		BY &BYVAR VAR_GROUP;
		WEIGHT &REPLI_ROOT&I;
		OUTPUT OUT=MEAN_TEMP MEAN=STAT1 VAR=STAT2;
	RUN;

	DATA MEAN_TEMP;
		SET MEAN_TEMP;
		L=&I;
	RUN;

	PROC APPEND BASE = BRR_TEMP1 DATA=MEAN_TEMP;
 	RUN;

%END;

PROC SORT DATA=BRR_TEMP1;
	BY &BYVAR L VAR_GROUP;
RUN;
PROC TRANSPOSE DATA=BRR_TEMP1 OUT=BRR_TEMP2 PREFIX=M;
	VAR STAT1	;
	BY &BYVAR L;
	ID VAR_GROUP;
RUN;
PROC TRANSPOSE DATA=BRR_TEMP1 OUT=BRR_TEMP3 PREFIX=V;
	VAR STAT2	;
	BY &BYVAR L;
	ID VAR_GROUP;
RUN;
DATA BRR_TEMP4;
	MERGE BRR_TEMP2 BRR_TEMP3;
	BY 	&BYVAR L;
	PV=(M1-M2)/(((V1+V2)/2)**0.5);
	KEEP  &BYVAR L PV;
RUN;
	

DATA BRR_TEMP5(KEEP=&BYVAR PV)BRR_TEMP6(KEEP=&BYVAR STAT);
	SET BRR_TEMP4;
	IF L > 0 THEN OUTPUT BRR_TEMP5;     
	ELSE DO;
		STAT =PV;
		OUTPUT BRR_TEMP6;
	END;
RUN;

PROC SORT DATA=BRR_TEMP5;
	BY &BYVAR;
RUN;

PROC SORT DATA=BRR_TEMP6;
	BY &BYVAR;
RUN;

DATA BRR_TEMP7;
	MERGE BRR_TEMP5 BRR_TEMP6;
	BY &BYVAR;
	VARI=((PV-STAT)**2)*(1/20);
RUN;

PROC UNIVARIATE DATA=BRR_TEMP7 NOPRINT;
	VAR VARI;
	BY &BYVAR;
	OUTPUT OUT=BRR_TEMP8 SUM=SS;
RUN;


DATA &OUTFILE;
	MERGE BRR_TEMP6 BRR_TEMP8;
	BY &BYVAR;
	SESTAT=(SS)**0.5;
	FORMAT STAT F10.2;
	FORMAT SESTAT F10.2;
	KEEP &BYVAR STAT SESTAT;
RUN;


PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRRDATA MEAN_TEMP BRR_TEMP1 BRR_TEMP2 BRR_TEMP3 BRR_TEMP4 BRR_TEMP5 BRR_TEMP6 BRR_TEMP7 BRR_TEMP8;
RUN;

OPTIONS NOTES;

%MEND;
