
%MACRO BRR_EFFECT_PV(INFILE=,
				     REPLI_ROOT=,	
				     BYVAR=,
				     PV_ROOT=,
				     EFFECT=,
				     OUTFILE=);

/* 

MEANING OF THE MACRO ARGUMENTS

INFILE		= 	INPUT DATA FILE.
REPLI_ROOT	= 	ROOT OF THE FINAL WEIGHT AND 80 REPLICATES VARIABLE NAMES. 
				FINAL WEIGHT VARIABLE NAME MUST BE THE REPLICATION ROOT FOLLOWED BY 0.
BYVAR		= 	BREAKDOWN VARIABLES
PV_ROOT		= 	ROOT OF THE 5 PLAUSIBLE VALUES VARIABLES NAMES
EFFECT		=	THIS ARGUMENT HAS THREE COMPONENTS: THE VARIABLE NAME AND THE TWO CATEGORIES ON WHICH THE EFFECT
				SIZE WILL BE COMPUTED. THE EFFECT SIZE WILL BE THE SECOND COMPONENT MINUS THE THIRD COMPONENT
OUTFILE		= 	FILE WITH THE STATISTIC ESTIMATES AND THEIR STANDARD ERROR ESTIMATE

*/ 

%LET GROUP=%SCAN(&EFFECT,1);
%LET CAT1=%SCAN(&EFFECT,2);
%LET CAT2=%SCAN(&EFFECT,3);

OPTIONS NONOTES;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP1;
RUN;

DATA BRRDATA;
	SET &INFILE;
	IF (&GROUP=&CAT1) THEN VAR_GROUP=1;
	IF (&GROUP=&CAT2) THEN VAR_GROUP=2;
	IF (VAR_GROUP IN (1,2));
	KEEP &REPLI_ROOT.0-&REPLI_ROOT.80 &PV_ROOT.1-&PV_ROOT.5 &BYVAR  VAR_GROUP;
RUN;

PROC SORT DATA=BRRDATA;
  BY &BYVAR VAR_GROUP;
RUN;

%DO I = 0 %TO 80;
 	PROC MEANS DATA=BRRDATA VARDEF=WGT NOPRINT;
		VAR &PV_ROOT.1 - &PV_ROOT.5  ;
		BY &BYVAR VAR_GROUP;
		WEIGHT &REPLI_ROOT&I;
		OUTPUT OUT=MEAN_TEMP MEAN=MOY1-MOY5 VAR=VARI1-VARI5;
	RUN;

	DATA MEAN_TEMP;
		SET MEAN_TEMP;
		L=&I;
	RUN;

	PROC APPEND BASE = BRR_TEMP1 DATA=MEAN_TEMP;
 	RUN;

%END;

PROC SORT DATA=BRR_TEMP1;
	BY &BYVAR L VAR_GROUP;
RUN;
PROC TRANSPOSE DATA=BRR_TEMP1 (RENAME=(MOY1=V1 MOY2=V2 MOY3=V3 MOY4=V4 MOY5=V5)) OUT=BRR_TEMP2 PREFIX=M;
	VAR V1-V5	;
	BY &BYVAR L;
	ID VAR_GROUP;
RUN;
PROC TRANSPOSE DATA=BRR_TEMP1 (RENAME=(VARI1=V1 VARI2=V2 VARI3=V3 VARI4=V4 VARI5=V5))OUT=BRR_TEMP3 PREFIX=V;
	VAR V1-V5	;
	BY &BYVAR L;
	ID VAR_GROUP;
RUN;
DATA BRR_TEMP4;
	MERGE BRR_TEMP2 BRR_TEMP3;
	BY 	&BYVAR L _NAME_;
	PV=(M1-M2)/(((V1+V2)/2)**0.5);
	KEEP  &BYVAR L PV _NAME_;
RUN;
PROC TRANSPOSE DATA=BRR_TEMP4 OUT=BRR_TEMP5(DROP=_NAME_) PREFIX=P;
	VAR PV;
	BY &BYVAR L;
	ID _NAME_;
RUN;

DATA BRR_TEMP6(DROP=STAT FIN1-FIN5 MESVAR) BRR_TEMP7(KEEP=&BYVAR STAT FIN1-FIN5 MESVAR);
 	SET BRR_TEMP5;
    IF L > 0 THEN OUTPUT BRR_TEMP6;     
    ELSE DO;
       	STAT =(PV1+PV2+PV3+PV4+PV5)/5;
	   	FIN1=PV1;
	   	FIN2=PV2;
	   	FIN3=PV3;
	   	FIN4=PV4;
	   	FIN5=PV5;
       	MESVAR=(((STAT-FIN1)**2)+((STAT-FIN2)**2)+((STAT-FIN3)**2)+((STAT-FIN4)**2)+((STAT-FIN5)**2))/4;
      	OUTPUT BRR_TEMP7;
   	END;
RUN;

PROC SORT DATA=BRR_TEMP6;
 	BY &BYVAR;
RUN;

PROC SORT DATA=BRR_TEMP7;
 	BY &BYVAR;
RUN;

DATA BRR_TEMP8;
	MERGE BRR_TEMP6 BRR_TEMP7;
	BY &BYVAR;
	ARRAY A (5)
		PV1-PV5;
	ARRAY B (5)
		FIN1-FIN5;
	ARRAY C(5)
		VAR1-VAR5;
	DO I=1 TO 5;
	C(I)=(1/20)*((A(I)-B(I))**2);
	END;
RUN;

PROC UNIVARIATE DATA=BRR_TEMP8 NOPRINT;
	VAR VAR1 VAR2 VAR3 VAR4 VAR5;
	BY &BYVAR;
	OUTPUT OUT=BRR_TEMP9 SUM=SS1 SS2 SS3 SS4 SS5;
RUN;

DATA &OUTFILE;
	MERGE BRR_TEMP7 BRR_TEMP9;
	BY &BYVAR;
	SAMP=(SS1+SS2+SS3+SS4+SS5)/5;
	FINVAR=(SAMP+(1.2*MESVAR));
	SESTAT=(FINVAR)**0.5;
	FORMAT STAT F10.2;
	FORMAT SESTAT F10.2;
	KEEP &BYVAR STAT SESTAT  ;
RUN;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRRDATA MEAN_TEMP BRR_TEMP1 BRR_TEMP2 BRR_TEMP3 BRR_TEMP4 BRR_TEMP5 BRR_TEMP6 BRR_TEMP7 BRR_TEMP8 BRR_TEMP9;
RUN;

OPTIONS NOTES;

%MEND;
