%macro BRR_FREQ(INFILE=,
				REPLI_ROOT=,
				BYVAR=,
				VAR=,
				LIMIT=,
				LIMIT_CRITERIA=,
				ID_SCHOOL=,
				OUTFILE=);

/* 

MEANING OF THE MACRO ARGUMENTS

INFILE			= 	INPUT DATA FILE.
REPLI_ROOT		= 	ROOT OF THE FINAL WEIGHT AND 80 REPLICATES VARIABLE NAMES. 
					FINAL WEIGHT VARIABLE NAME MUST BE THE REPLICATION ROOT FOLLOWED BY 0.
BYVAR			= 	BREAKDOWN VARIABLES
VAR				= 	VARIABLES ON WHICH PERCENTAGES WILL BE COMPUTED
LIMIT          	=    FLAGGING  YES OR NO
LIMIT_CRITERIA 	=   1) NUMBER OF STUDENTS 2) NUMBER OF SCHOOLS 3) PERCENTAGE OF STUDENTS AND 4) NUMBER OF VARIABLES FROM 
					THE BYVAR ARGUMENT FOR DEFINING THE POPULATION OF REFERENCE 
ID_SCHOOL	   	=	VARIABLE NAME FOR THE SCHOOL IDENTIFICATION
OUTFILE			= 	FILE WITH THE STATISTIC ESTIMATES AND THEIR STANDARD ERROR ESTIMATE

*/ 

OPTIONS NONOTES;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP0;
RUN;

PROC SORT DATA=&INFILE
	OUT=BRRDATA(KEEP=&REPLI_ROOT.0-&REPLI_ROOT.80 &BYVAR &VAR &ID_SCHOOL);
	BY &BYVAR &VAR;
RUN;

%DO I = 0 %TO 80;
	PROC FREQ DATA=BRRDATA NOPRINT;
		TABLE &VAR /OUT=FREQ_TEMP ;
		BY &BYVAR;
		WEIGHT &REPLI_ROOT&I;
	RUN;

	DATA FREQ_TEMP;
		SET FREQ_TEMP;
		L=&I;
		KEEP &BYVAR L &VAR PERCENT;
	RUN;

	PROC APPEND BASE = BRR_TEMP0 DATA=FREQ_TEMP;
	RUN;
%END;

PROC SORT DATA=BRR_TEMP0 OUT=BRR_TEMP1 (RENAME=(PERCENT=PV));
	BY &BYVAR L &VAR;
RUN;

DATA BRR_TEMP2(KEEP=&BYVAR &VAR PV)BRR_TEMP3(KEEP=&BYVAR &VAR STAT);
	SET BRR_TEMP1;
	IF L > 0 THEN OUTPUT BRR_TEMP2;     
	ELSE DO;
		STAT =PV;
		OUTPUT BRR_TEMP3;
	END;
RUN;

PROC SORT DATA=BRR_TEMP2;
	BY &BYVAR &VAR;
RUN;

PROC SORT DATA=BRR_TEMP3;
	BY &BYVAR &VAR;
RUN;

DATA BRR_TEMP4;
	MERGE BRR_TEMP2 BRR_TEMP3;
	BY &BYVAR &VAR;
	VARI=((PV-STAT)**2)*(1/20);
RUN;

PROC UNIVARIATE DATA=BRR_TEMP4 NOPRINT;
	VAR VARI;
	BY &BYVAR &VAR;
	OUTPUT OUT=BRR_TEMP5 SUM=SS;
RUN;

DATA BRR_TEMP6;
	MERGE BRR_TEMP3 BRR_TEMP5;
	BY &BYVAR &VAR;
	SESTAT=(SS)**0.5;
	FORMAT STAT F10.2;
	FORMAT SESTAT F10.2;
	KEEP &BYVAR &VAR STAT SESTAT;
RUN;

%IF (%UPCASE(&LIMIT)=NO) %THEN %DO;

	DATA &OUTFILE;
		SET BRR_TEMP6;
	RUN;

%END;
%ELSE %DO;

	PROC FREQ DATA=BRRDATA NOPRINT;
		TABLE &VAR /OUT=BRR_TEMP7;
		BY &BYVAR;
	RUN;
	%LET FLAG_STUD=%SCAN(&LIMIT_CRITERIA,1);
	DATA BRR_TEMP7;
		SET BRR_TEMP7;
		IF (COUNT < &FLAG_STUD) THEN FLAG_STUD=1;
		ELSE FLAG_STUD=0;
		KEEP &BYVAR &VAR FLAG_STUD;
	RUN;
	PROC SORT DATA=BRRDATA;
		BY &BYVAR  &ID_SCHOOL;
	RUN;
	PROC FREQ DATA=BRRDATA NOPRINT;
		TABLE &VAR /OUT=BRR_TEMP8;
		BY &BYVAR  &ID_SCHOOL;
	RUN;		
	PROC FREQ DATA=BRR_TEMP8 NOPRINT;
		TABLE &VAR /OUT=BRR_TEMP9;
		BY &BYVAR;
	RUN;
	%LET FLAG_SCH=%SCAN(&LIMIT_CRITERIA,2);
	DATA BRR_TEMP9;
		SET BRR_TEMP9;
		IF (COUNT < &FLAG_SCH) THEN FLAG_SCH=1;
		ELSE FLAG_SCH=0;
		KEEP &BYVAR &VAR FLAG_SCH;
	RUN;

	PROC FREQ DATA=BRRDATA NOPRINT;
		TABLE &VAR/OUT=BRR_TEMP10;
		BY &BYVAR;
		WEIGHT &REPLI_ROOT.0;
	RUN;
	%LET K=1;
	%LET POPREF=;
	%LET NBVAR=%SCAN(&LIMIT_CRITERIA,4);
	%DO %WHILE(&K <= &NBVAR);
		%LET POPREF_ADD=%SCAN(&BYVAR,&K);
		%LET POPREF=&POPREF &POPREF_ADD;
		%LET K=%EVAL(&K+1);
	%END;
	
	PROC MEANS DATA=BRR_TEMP10 NOPRINT;
		VAR COUNT;
		BY &POPREF;
		OUTPUT OUT=BRR_TEMP11 SUM=SOMWGT;
	RUN;
	%LET FLAG_PCT=%SCAN(&LIMIT_CRITERIA,3);
	DATA BRR_TEMP12;
		MERGE BRR_TEMP10 BRR_TEMP11;
		BY &POPREF;
		PCT=(COUNT/SOMWGT)*100;
		IF (PCT < &FLAG_PCT) THEN FLAG_PCT=1;
		ELSE FLAG_PCT=0;
		KEEP &BYVAR &VAR FLAG_PCT;
	RUN;
	DATA &OUTFILE;
		MERGE BRR_TEMP6 BRR_TEMP7 BRR_TEMP9 BRR_TEMP12;
		BY &BYVAR &VAR;
		IF (STAT NE .); 
	RUN;
		

	PROC DATASETS LIBRARY=WORK NOLIST;
		DELETE BRR_TEMP7 BRR_TEMP8 BRR_TEMP9 BRR_TEMP10 BRR_TEMP11 BRR_TEMP12;
	RUN;
%END;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP0 BRR_TEMP1 BRR_TEMP2 BRR_TEMP3 BRR_TEMP4 BRR_TEMP5 BRR_TEMP6 FREQ_TEMP BRRDATA;
RUN;

OPTIONS NOTES;

%MEND BRR_FREQ;
