%macro BRR_FREQ_PV(INFILE=,
				   REPLI_ROOT=,
				   BYVAR=,
				   PV_ROOT=,
				   LIMIT=,
				   LIMIT_CRITERIA=,
				   ID_SCHOOL=,
				   OUTFILE=);

/* 

MEANING OF THE MACRO ARGUMENTS

INFILE			= 	INPUT DATA FILE.
REPLI_ROOT		= 	ROOT OF THE FINAL WEIGHT AND 80 REPLICATES VARIABLE NAMES. 
					FINAL WEIGHT VARIABLE NAME MUST BE THE REPLICATION ROOT FOLLOWED BY 0.
BYVAR			= 	BREAKDOWN VARIABLES
PV_ROOT			= 	ROOT OF THE 5 PROFICIENCY LEVEL VARIABLES NAMES
LIMIT          	=   FLAGGING  YES OR NO
LIMIT_CRITERIA 	=   1) NUMBER OF STUDENTS 2) NUMBER OF SCHOOLS 3) PERCENTAGE OF STUDENTS AND 4) NUMBER OF VARIABLES FROM 
					THE BYVAR ARGUMENT FOR DEFINING THE POPULATION OF REFERENCE 
ID_SCHOOL	   	=	VARIABLE NAME FOR THE SCHOOL IDENTIFICATION
OUTFILE			= 	FILE WITH THE STATISTIC ESTIMATES AND THEIR STANDARD ERROR ESTIMATE

*/ 

OPTIONS NONOTES;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP0;
RUN;

%DO I = 1 %TO 5;

	PROC SORT DATA=&INFILE
		OUT=BRRDATA(KEEP=&REPLI_ROOT.0-&REPLI_ROOT.80 &BYVAR &PV_ROOT.1-&PV_ROOT.5 &ID_SCHOOL);
		BY &BYVAR &PV_ROOT&I;
	RUN;

	%DO J = 0 %TO 80;

		PROC FREQ DATA=BRRDATA NOPRINT;
			TABLE &PV_ROOT&I /OUT=FREQ_TEMP ;
			BY &BYVAR;
			WEIGHT &REPLI_ROOT&J;
		RUN;

		DATA FREQ_TEMP;
			SET FREQ_TEMP;
			K=&I;
			L=&J;
			&PV_ROOT=&PV_ROOT&I;
			KEEP &BYVAR K L PERCENT &PV_ROOT;
		RUN;

		PROC APPEND BASE = BRR_TEMP0 DATA=FREQ_TEMP;
		RUN;

	%END;

%END;

PROC SORT DATA=BRR_TEMP0;
	BY &BYVAR L &PV_ROOT;
RUN;

PROC TRANSPOSE DATA=BRR_TEMP0 OUT=BRR_TEMP1 PREFIX=PV;
	BY &BYVAR L &PV_ROOT;
	VAR PERCENT;
RUN; 

DATA BRR_TEMP1;
	SET BRR_TEMP1;
	DROP _NAME_ _LABEL_;
RUN;

DATA BRR_TEMP2(DROP=STAT FIN1-FIN5 MESVAR)BRR_TEMP3(KEEP=&BYVAR &PV_ROOT STAT FIN1-FIN5 MESVAR);
	SET BRR_TEMP1;
	IF L > 0 THEN OUTPUT BRR_TEMP2;     
	ELSE DO;
		STAT =(PV1+PV2+PV3+PV4+PV5)/5;
		FIN1=PV1;
		FIN2=PV2;
		FIN3=PV3;
		FIN4=PV4;
		FIN5=PV5;
		MESVAR=(((STAT-FIN1)**2)+((STAT-FIN2)**2)+((STAT-FIN3)**2)+((STAT-FIN4)**2)+((STAT-FIN5)**2))/4;
		OUTPUT BRR_TEMP3;
	END;
RUN;

PROC SORT DATA=BRR_TEMP2;
	BY &BYVAR &PV_ROOT;
RUN;

PROC SORT DATA=BRR_TEMP3;
	BY &BYVAR &PV_ROOT;
RUN;

DATA BRR_TEMP4;
	MERGE BRR_TEMP2 BRR_TEMP3;
	BY &BYVAR &PV_ROOT;
	ARRAY A (5)
		PV1-PV5;
	ARRAY B (5)
		FIN1-FIN5;
	ARRAY C(5)
		VAR1-VAR5;
	DO I=1 TO 5;
		C(I)=(1/20)*((A(I)-B(I))**2);
	END;
RUN;

PROC UNIVARIATE DATA=BRR_TEMP4 NOPRINT;
	VAR VAR1 VAR2 VAR3 VAR4 VAR5;
	BY &BYVAR &PV_ROOT;
	OUTPUT OUT=BRR_TEMP5 SUM=SS1 SS2 SS3 SS4 SS5;
RUN;

DATA BRR_TEMP6;
	MERGE BRR_TEMP3 BRR_TEMP5;
	BY &BYVAR &PV_ROOT;
	SAMP=(SS1+SS2+SS3+SS4+SS5)/5;
	FINVAR=(SAMP+(1.2*MESVAR));
	SESTAT=(FINVAR)**0.5;
	KEEP &BYVAR &PV_ROOT STAT SESTAT;
RUN;

%IF (%UPCASE(&LIMIT)=NO) %THEN %DO;

	DATA &OUTFILE;
		SET BRR_TEMP6;
	RUN;

%END;

%ELSE %DO;
	%DO M=1 %TO 5;
	PROC FREQ DATA=BRRDATA NOPRINT;
		TABLE &PV_ROOT&M/OUT=BRR_TEMP7;
		BY &BYVAR;
	RUN;
	DATA BRR_TEMP8;
		SET BRR_TEMP7;
		COUNT&M=COUNT;
		&PV_ROOT=&PV_ROOT&M;
		KEEP &BYVAR &PV_ROOT COUNT&M;
	RUN;
	%IF (&M=1) %THEN %DO;
		DATA BRR_TEMP9;
			SET BRR_TEMP8;
		RUN;
	%END;
	%ELSE %DO;
		DATA BRR_TEMP9;
			MERGE BRR_TEMP9 BRR_TEMP8;
			BY &BYVAR &PV_ROOT;
		RUN;
	%END;
	%END;
	%LET FLAG_STUD=%SCAN(&LIMIT_CRITERIA,1);
	DATA BRR_TEMP9;
		SET BRR_TEMP9;
		COUNT=(COUNT1+COUNT2+COUNT3+COUNT4+COUNT5)/5;
		IF (COUNT < &FLAG_STUD) THEN FLAG_STUD=1;
		ELSE FLAG_STUD=0;
		KEEP &BYVAR &PV_ROOT FLAG_STUD;
	RUN;

	PROC SORT DATA=BRRDATA;
		BY &BYVAR  &ID_SCHOOL;
	RUN;

	%DO M=1 %TO 5;
	PROC FREQ DATA=BRRDATA NOPRINT;
		TABLE &PV_ROOT&M /OUT=BRR_TEMP10;
		BY &BYVAR  &ID_SCHOOL;
	RUN;	
	PROC FREQ DATA=BRR_TEMP10 NOPRINT;
		TABLE &PV_ROOT&M  /OUT=BRR_TEMP11;
		BY &BYVAR;
	RUN;
	DATA BRR_TEMP12;
		SET BRR_TEMP11;
		COUNT&M=COUNT;
		&PV_ROOT=&PV_ROOT&M;
		KEEP &BYVAR &PV_ROOT COUNT&M;
	RUN;
	%IF (&M=1) %THEN %DO;
		DATA BRR_TEMP13;
			SET BRR_TEMP12;
		RUN;
	%END;
	%ELSE %DO;
		DATA BRR_TEMP13;
			MERGE BRR_TEMP13 BRR_TEMP12;
			BY &BYVAR &PV_ROOT;
		RUN;
	%END;
	%END;
	%LET FLAG_SCH=%SCAN(&LIMIT_CRITERIA,2);
	DATA BRR_TEMP13;
		SET BRR_TEMP13;
		COUNT=(COUNT1+COUNT2+COUNT3+COUNT4+COUNT5)/5;
		IF (COUNT < &FLAG_SCH) THEN FLAG_SCH=1;
		ELSE FLAG_SCH=0;
		KEEP &BYVAR &PV_ROOT FLAG_SCH;
	RUN;

	%DO M=1 %TO 5;
	PROC FREQ DATA=BRRDATA NOPRINT;
		TABLE &PV_ROOT&M/OUT=BRR_TEMP14;
		BY &BYVAR;
		WEIGHT &REPLI_ROOT.0;
	RUN;
	DATA BRR_TEMP15;
		SET BRR_TEMP14;
		COUNT&M=COUNT;
		&PV_ROOT=&PV_ROOT&M;
		KEEP &BYVAR &PV_ROOT COUNT&M;
	RUN;
	%IF (&M=1) %THEN %DO;
		DATA BRR_TEMP16;
			SET BRR_TEMP15;
		RUN;
	%END;
	%ELSE %DO;
		DATA BRR_TEMP16;
			MERGE BRR_TEMP16 BRR_TEMP15;
			BY &BYVAR &PV_ROOT;
		RUN;
	%END;
	%END;

	%LET K=1;
	%LET POPREF=;
	%LET NBVAR=%SCAN(&LIMIT_CRITERIA,4);
	%DO %WHILE(&K <= &NBVAR);
		%LET POPREF_ADD=%SCAN(&BYVAR,&K);
		%LET POPREF=&POPREF &POPREF_ADD;
		%LET K=%EVAL(&K+1);
	%END;

	PROC MEANS DATA=BRR_TEMP16 NOPRINT;
		VAR COUNT1-COUNT5 ;
		BY &POPREF;
		OUTPUT OUT=BRR_TEMP17 SUM=SOMWGT1-SOMWGT5;
	RUN;
	%LET FLAG_PCT=%SCAN(&LIMIT_CRITERIA,3);
	DATA BRR_TEMP18;
		MERGE BRR_TEMP16 BRR_TEMP17;
		BY &POPREF;
		ARRAY B1 (5) PCT1-PCT5;
		ARRAY B2 (5) COUNT1-COUNT5;
		ARRAY B3 (5) SOMWGT1-SOMWGT5;
		DO A=1 TO 5;
			B1(A)=(B2(A)/B3(A))*100;
		END;
		PCT=(PCT1+PCT2+PCT3+PCT4+PCT5)/5;
		IF (PCT < &FLAG_PCT) THEN FLAG_PCT=1;
		ELSE FLAG_PCT=0;
		KEEP &BYVAR &PV_ROOT FLAG_PCT;
	RUN;
	
	DATA &OUTFILE;
		MERGE BRR_TEMP6 BRR_TEMP9 BRR_TEMP13 BRR_TEMP18;
		BY &BYVAR &PV_ROOT;
	RUN;
		
	PROC DATASETS LIBRARY=WORK NOLIST;
		DELETE BRR_TEMP7 BRR_TEMP8 BRR_TEMP9 BRR_TEMP10 BRR_TEMP11 BRR_TEMP12 BRR_TEMP13
				BRR_TEMP14 BRR_TEMP15 BRR_TEMP16 BRR_TEMP17 BRR_TEMP18;
	RUN;

%END;


PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP0 BRR_TEMP1 BRR_TEMP2 BRR_TEMP3 BRR_TEMP4 BRR_TEMP5 FREQ_TEMP BRR_TEMP6 MEAN_TEMP BRRDATA;
RUN;

OPTIONS NOTES;

%MEND BRR_FREQ_PV;
