%macro BRR_MIXED(INFILE=,
				 REPLI_ROOT=,
				 VARDEP=,
				 FIXEF=,
				 RANEF=,
				 BYVAR=,
				 LEVEL2=,
				 OUTSCREEN=,
				 OUTFILE=);

/* 

MEANING OF THE MACRO ARGUMENTS

INFILE		= 	INPUT DATA FILE.
REPLI_ROOT	= 	ROOT OF THE FINAL WEIGHT AND 80 REPLICATES VARIABLE NAMES. 
				FINAL WEIGHT VARIABLE NAME MUST BE THE REPLICATION ROOT FOLLOWED BY 0.
VARDEP		= 	DEPENDENT VARIABLES
FIXEF		=	LIST OF INDEPENDENT VARIABLES WITH FIXED EFFECT
RANEF		=	LIST OF INDEPENDENT VARIABLES WITH RANDOM EFFECT
LEVEL2		=	LEVEL 2 IDENTIFICATION VARIABLE
BYVAR		= 	BREAKDOWN VARIABLES
OUTSCREEN	=	DOS ADRESS FOR EXPORTING LISTING
OUTFILE		= 	FILE WITH THE STATISTIC ESTIMATES AND THEIR STANDARD ERROR ESTIMATE

*/ 


OPTION NONOTES CLEANUP;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP0;
RUN;

PROC SORT DATA=&INFILE
	OUT=BRR_PREP1(KEEP=&REPLI_ROOT.0-&REPLI_ROOT.80  &VARDEP &BYVAR &FIXEF &RANEF &LEVEL2);
	BY &BYVAR;
RUN;

%LET I=1;
%DO %WHILE(%LENGTH(%SCAN(&FIXEF,&I)));
	%LET I=%EVAL(&I+1);
%END;
%LET NB_FIXE=%EVAL(&I-1);


%LET I=1;
%DO %WHILE(%LENGTH(%SCAN(&RANEF,&I)));
	%LET I=%EVAL(&I+1);
%END;
%LET NB_RAN=%EVAL(&I-1);
%LET NB_TOT=%EVAL(&NB_FIXE+&NB_RAN+1);

DATA BRR_PREP2;
		SET BRR_PREP1;
		NB_MISS=0;
		ARRAY LIST_VAR (&NB_TOT) &VARDEP &FIXEF &RANEF;
		DO K=1 TO &NB_TOT;
			IF (LIST_VAR(K) IN (.,.I,.M,.N)) THEN NB_MISS=NB_MISS+1;
		END;
		IF (NB_MISS>1) THEN NB_MISS=1;
RUN;

PROC FREQ DATA=BRR_PREP2 NOPRINT;
	TABLE NB_MISS/OUT=&OUTFILE._DELETION;
	BY &BYVAR;
	WEIGHT &REPLI_ROOT.0;
RUN;

PROC UNIVARIATE DATA=BRR_PREP2 NOPRINT;
	VAR &REPLI_ROOT.0;
	BY &BYVAR;
	WHERE (NB_MISS=0);
	OUTPUT OUT=BRR_PREP3 N=NBRE SUM=SOMWGT;
RUN;

DATA BRRDATA;
	MERGE BRR_PREP2 BRR_PREP3;
	BY &BYVAR;
	ARRAY OLDWGT (81)&REPLI_ROOT.0-&REPLI_ROOT.80;
    ARRAY NEWWGT (81) STD_WGT0-STD_WGT80;
	DO I=1 TO 81;
		NEWWGT(I)=(OLDWGT(I)/SOMWGT)*NBRE;
	END;
RUN;

%DO J = 0 %TO 80;

		PROC MIXED DATA=BRRDATA NOCLPRINT NOITPRINT NOINFO METHOD=ML;
			CLASS &LEVEL2;
			MODEL &VARDEP=&FIXEF &RANEF /SOLUTION;
			RANDOM INTERCEPT &RANEF /SUBJECT=&LEVEL2;
			WEIGHT STD_WGT&J;
			BY &BYVAR;
			ODS OUTPUT SOLUTIONF=FIXE_TEMP COVPARMS=COV_TEMP;
		RUN;

		DATA FIXE_TEMP;
			SET FIXE_TEMP;
			L=&J;
			KEEP &BYVAR L EFFECT ESTIMATE ;
		RUN;
		PROC SORT DATA=FIXE_TEMP;
			by &BYVAR L EFFECT;
		RUN;
		DATA COV_TEMP;
			SET COV_TEMP;
			L=&J;
			KEEP &BYVAR L COVPARM ESTIMATE ;
		RUN;
		PROC SORT DATA=COV_TEMP;
			BY &BYVAR L COVPARM;
		RUN;

		%IF (&J=0) %then %do;
			DATA BRR_TEMP_FIXE;
				SET FIXE_TEMP;
			RUN;
			DATA BRR_TEMP_VAR;
				SET COV_TEMP;
			RUN; 
		%END;

		DATA BRR_TEMP_FIXE;
			MERGE BRR_TEMP_FIXE FIXE_TEMP;
			BY &BYVAR L EFFECT; 
 		RUN;
		DATA BRR_TEMP_VAR;
			MERGE BRR_TEMP_VAR COV_TEMP;
			BY &BYVAR L COVPARM;
		RUN;
	
%END;

/* FIXE PARAMETERS */

PROC SORT DATA=BRR_TEMP_FIXE OUT=BRR_TEMP1;
	BY &BYVAR L EFFECT;
RUN;

DATA BRR_TEMP2(KEEP=&BYVAR EFFECT ESTIMATE) BRR_TEMP3(KEEP=&BYVAR EFFECT STAT);
	SET BRR_TEMP1;
	IF L > 0 THEN OUTPUT BRR_TEMP2;     
	ELSE DO;
		STAT =ESTIMATE;
		OUTPUT BRR_TEMP3;
	END;
RUN;

PROC SORT DATA=BRR_TEMP2;
	BY &BYVAR EFFECT;
RUN;

PROC SORT DATA=BRR_TEMP3;
	BY &BYVAR EFFECT;
RUN;

DATA BRR_TEMP4;
	MERGE BRR_TEMP2 BRR_TEMP3;
	BY &BYVAR EFFECT;
	VARI=((ESTIMATE-STAT)**2)*(1/20);
RUN;

PROC UNIVARIATE DATA=BRR_TEMP4 NOPRINT;
	VAR VARI;
	BY &BYVAR EFFECT;
	OUTPUT OUT=BRR_TEMP5 SUM=SS;
RUN;

DATA &OUTFILE._FIXE;
	MERGE BRR_TEMP3 BRR_TEMP5;
	BY &BYVAR EFFECT;
	SESTAT=(SS)**0.5;
	FORMAT STAT F10.2;
	FORMAT SESTAT F10.2;
	KEEP &BYVAR EFFECT STAT SESTAT;
RUN;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_PREP1 BRR_PREP2 BRR_PREP3 BRR_TEMP_FIXE FIXE_TEMP BRR_TEMP1 BRR_TEMP2 BRR_TEMP3 BRR_TEMP4 BRR_TEMP5 MEAN_TEMP BRRDATA;
RUN;

/* RANDOM PARAMETERS */

PROC SORT DATA=BRR_TEMP_VAR OUT=BRR_TEMP1;
	BY &BYVAR L COVPARM;
RUN;

DATA BRR_TEMP2(KEEP=&BYVAR COVPARM ESTIMATE) BRR_TEMP3(KEEP=&BYVAR COVPARM STAT);
	SET BRR_TEMP1;
	IF L > 0 THEN OUTPUT BRR_TEMP2;     
	ELSE DO;
		STAT =ESTIMATE;
		OUTPUT BRR_TEMP3;
	END;
RUN;

PROC SORT DATA=BRR_TEMP2;
	BY &BYVAR COVPARM;
RUN;

PROC SORT DATA=BRR_TEMP3;
	BY &BYVAR COVPARM;
RUN;

DATA BRR_TEMP4;
	MERGE BRR_TEMP2 BRR_TEMP3;
	BY &BYVAR COVPARM;
	VARI=((ESTIMATE-STAT)**2)*(1/20);
RUN;

PROC UNIVARIATE DATA=BRR_TEMP4 NOPRINT;
	VAR VARI;
	BY &BYVAR COVPARM;
	OUTPUT OUT=BRR_TEMP5 SUM=SS;
RUN;

DATA &OUTFILE._VARIANCE;
	MERGE BRR_TEMP3 BRR_TEMP5;
	BY &BYVAR COVPARM;
	SESTAT=(SS)**0.5;
	FORMAT STAT F10.2;
	FORMAT SESTAT F10.2;
	KEEP &BYVAR COVPARM STAT SESTAT;
RUN;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE COV_TEMP  BRR_TEMP1 BRR_TEMP2 BRR_TEMP3 BRR_TEMP4 BRR_TEMP5 MEAN_TEMP BRRDATA;
RUN;

/* INTRACLASS CORRELATION */

PROC SORT DATA=BRR_TEMP_VAR ;
	BY &BYVAR L COVPARM;
RUN;

PROC TRANSPOSE DATA=BRR_TEMP_VAR OUT=BRR_TEMP0;
	VAR ESTIMATE;
	BY &BYVAR L;
	ID COVPARM;
RUN;

DATA BRR_TEMP1;
	SET BRR_TEMP0;
	ESTIMATE=(INTERCEPT/(INTERCEPT+RESIDUAL));
	KEEP &BYVAR L ESTIMATE;
RUN;

DATA BRR_TEMP2(KEEP=&BYVAR ESTIMATE) BRR_TEMP3(KEEP=&BYVAR STAT);
	SET BRR_TEMP1;
	IF L > 0 THEN OUTPUT BRR_TEMP2;     
	ELSE DO;
		STAT =ESTIMATE;
		OUTPUT BRR_TEMP3;
	END;
RUN;

PROC SORT DATA=BRR_TEMP2;
	BY &BYVAR;
RUN;

PROC SORT DATA=BRR_TEMP3;
	BY &BYVAR ;
RUN;

DATA BRR_TEMP4;
	MERGE BRR_TEMP2 BRR_TEMP3;
	BY &BYVAR ;
	VARI=((ESTIMATE-STAT)**2)*(1/20);
RUN;

PROC UNIVARIATE DATA=BRR_TEMP4 NOPRINT;
	VAR VARI;
	BY &BYVAR ;
	OUTPUT OUT=BRR_TEMP5 SUM=SS;
RUN;

DATA &OUTFILE._INTRACLASS;
	MERGE BRR_TEMP3 BRR_TEMP5;
	BY &BYVAR ;
	SESTAT=(SS)**0.5;
	FORMAT STAT F10.2;
	FORMAT SESTAT F10.2;
	KEEP &BYVAR STAT SESTAT;
RUN;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE COV_TEMP  BRR_TEMP1 BRR_TEMP2 BRR_TEMP3 BRR_TEMP4 BRR_TEMP5 MEAN_TEMP BRRDATA;
RUN;


PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP_VAR COV_TEMP  BRR_TEMP0 BRR_TEMP1 BRR_TEMP2 BRR_TEMP3 BRR_TEMP4 BRR_TEMP5 MEAN_TEMP BRRDATA;
RUN;


OPTIONS NOTES;

%MEND;
