%macro BRR_MIXED_PV(INFILE=,
                                   REPLI_ROOT=,
                                   PV_ROOT=,
                                   FIXEF=,
                                   RANEF=,
                                   BYVAR=,
                                   LEVEL2=,
                                   OUTSCREEN=,
                                   OUTFILE=,);

/*

MEANING OF THE MACRO ARGUMENTS

INFILE          =       INPUT DATA FILE.
REPLI_ROOT      =       ROOT OF THE FINAL WEIGHT AND 80 REPLICATES VARIABLE NAMES.
                                FINAL WEIGHT VARIABLE NAME MUST BE THE REPLICATION ROOT FOLLOWED BY 0.
PV_ROOT         =       ROOT OF THE 5 PROFICIENCY LEVEL VARIABLES NAMES
FIXEF           =       LIST OF INDEPENDENT VARIABLES WITH FIXED EFFECT
RANEF           =       LIST OF INDEPENDENT VARIABLES WITH RANDOM EFFECT
LEVEL2          =       LEVEL 2 IDENTIFICATION VARIABLE
BYVAR           =       BREAKDOWN VARIABLES
OUTSCREEN       =       DOS ADRESS FOR EXPORTING LISTING
OUTFILE         =       FILE WITH THE STATISTIC ESTIMATES AND THEIR STANDARD ERROR ESTIMATE

*/

OPTION NONOTES CLEANUP;


FILENAME MYOUTPUT &OUTSCREEN;
PROC PRINTTO PRINT=MYOUTPUT NEW;
RUN;

PROC DATASETS LIBRARY=WORK NOLIST;
        DELETE BRR_TEMP0;
RUN;

PROC SORT DATA=&INFILE
        OUT=BRR_PREP1(KEEP=&REPLI_ROOT.0-&REPLI_ROOT.80  &PV_ROOT.1-&PV_ROOT.5 &BYVAR &FIXEF &RANEF &LEVEL2);
        BY &BYVAR;
RUN;

%LET I=1;
%DO %WHILE(%LENGTH(%SCAN(&FIXEF,&I)));
        %LET I=%EVAL(&I+1);
%END;
%LET NB_FIXE=%EVAL(&I-1);


%LET I=1;
%DO %WHILE(%LENGTH(%SCAN(&RANEF,&I)));
        %LET I=%EVAL(&I+1);
%END;
%LET NB_RAN=%EVAL(&I-1);
%LET NB_TOT=%EVAL(&NB_FIXE+&NB_RAN+5);

DATA BRR_PREP2;
                SET BRR_PREP1;
                NB_MISS=0;
                ARRAY LIST_VAR (&NB_TOT) &PV_ROOT.1-&PV_ROOT.5 &FIXEF &RANEF;
                DO K=1 TO &NB_TOT;
                        IF (LIST_VAR(K) IN (.,.I,.M,.N)) THEN NB_MISS=NB_MISS+1;
                END;
                IF (NB_MISS>1) THEN NB_MISS=1;
RUN;

PROC FREQ DATA=BRR_PREP2 NOPRINT;
        TABLE NB_MISS/OUT=&OUTFILE._DELETION;
        BY &BYVAR;
        WEIGHT &REPLI_ROOT.0;
RUN;

PROC UNIVARIATE DATA=BRR_PREP2 NOPRINT;
        VAR &REPLI_ROOT.0;
        BY &BYVAR;
        WHERE (NB_MISS=0);
        OUTPUT OUT=BRR_PREP3 N=NBRE SUM=SOMWGT;
RUN;

DATA BRRDATA;
        MERGE BRR_PREP2 BRR_PREP3;
        BY &BYVAR;
        ARRAY OLDWGT (81)&REPLI_ROOT.0-&REPLI_ROOT.80;
    ARRAY NEWWGT (81) STD_WGT0-STD_WGT80;
        DO I=1 TO 81;
                NEWWGT(I)=(OLDWGT(I)/SOMWGT)*NBRE;
        END;
RUN;

%DO I = 1 %TO 5;

        %DO J = 0 %TO 80;

                PROC MIXED DATA=BRRDATA NOCLPRINT NOITPRINT NOINFO METHOD=ML;
                        CLASS &LEVEL2;
                        MODEL &PV_ROOT&I=&FIXEF &RANEF /SOLUTION;
                        RANDOM INTERCEPT &RANEF /SUBJECT=&LEVEL2;
                        WEIGHT STD_WGT&J;
                        BY &BYVAR;
                        ODS OUTPUT SOLUTIONF=FIXE_TEMP COVPARMS=COV_TEMP;
                RUN;

                DATA FIXE_TEMP;
                        SET FIXE_TEMP;
                        L=&J;
                        PV&I=ESTIMATE;
                        KEEP &BYVAR L EFFECT PV&I ;
                RUN;
                PROC SORT DATA=FIXE_TEMP;
                        by &BYVAR L EFFECT;
                RUN;
                DATA COV_TEMP;
                        SET COV_TEMP;
                        L=&J;
                        PV&I=ESTIMATE;
                        KEEP &BYVAR L COVPARM PV&I ;
                RUN;
                PROC SORT DATA=COV_TEMP;
                        BY &BYVAR L COVPARM;
                RUN;

                %IF (&I=1 AND &J=0) %then %do;
                        DATA BRR_TEMP_FIXE;
                                SET FIXE_TEMP;
                        RUN;
                        DATA BRR_TEMP_VAR;
                                SET COV_TEMP;
                        RUN;
                %END;

                DATA BRR_TEMP_FIXE;
                        MERGE BRR_TEMP_FIXE FIXE_TEMP;
                        BY &BYVAR L EFFECT;
                RUN;
                DATA BRR_TEMP_VAR;
                        MERGE BRR_TEMP_VAR COV_TEMP;
                        BY &BYVAR L COVPARM;
                RUN;

        %END;

%END;

PROC SORT DATA=BRR_TEMP_FIXE OUT=BRR_TEMP1;
        BY &BYVAR L EFFECT;
RUN;

DATA BRR_TEMP2(DROP=STAT FIN1-FIN5 MESVAR)BRR_TEMP3(KEEP=&BYVAR EFFECT STAT FIN1-FIN5 MESVAR);
        SET BRR_TEMP1;
        IF L > 0 THEN OUTPUT BRR_TEMP2;
        ELSE DO;
                STAT =(PV1+PV2+PV3+PV4+PV5)/5;
                FIN1=PV1;
                FIN2=PV2;
                FIN3=PV3;
                FIN4=PV4;
                FIN5=PV5;
                MESVAR=(((STAT-FIN1)**2)+((STAT-FIN2)**2)+((STAT-FIN3)**2)+((STAT-FIN4)**2)+((STAT-FIN5)**2))/4;
                OUTPUT BRR_TEMP3;
        END;
RUN;

PROC SORT DATA=BRR_TEMP2;
        BY &BYVAR EFFECT L;
RUN;

PROC SORT DATA=BRR_TEMP3;
        BY &BYVAR EFFECT;
RUN;

DATA BRR_TEMP4;
        MERGE BRR_TEMP2 BRR_TEMP3;
        BY &BYVAR EFFECT;
        ARRAY A (5)
                PV1-PV5;
        ARRAY B (5)
                FIN1-FIN5;
        ARRAY C(5)
                VAR1-VAR5;
        DO I=1 TO 5;
                C(I)=(1/20)*((A(I)-B(I))**2);
        END;
RUN;

PROC UNIVARIATE DATA=BRR_TEMP4 NOPRINT;
        VAR VAR1 VAR2 VAR3 VAR4 VAR5;
        BY &BYVAR EFFECT;
        OUTPUT OUT=BRR_TEMP5 SUM=SS1 SS2 SS3 SS4 SS5;
RUN;

DATA &OUTFILE._FIXE;
        MERGE BRR_TEMP3 BRR_TEMP5;
        BY &BYVAR EFFECT;
        SAMP=(SS1+SS2+SS3+SS4+SS5)/5;
        FINVAR=(SAMP+(1.2*MESVAR));
        SESTAT=(FINVAR)**0.5;
        FORMAT STAT F10.1;
        FORMAT SESTAT F10.2;
        KEEP &BYVAR EFFECT STAT SESTAT;
RUN;

PROC DATASETS LIBRARY=WORK NOLIST;
        DELETE BRR_TEMP_FIXE FIXE_TEMP BRR_TEMP1 BRR_TEMP2 BRR_TEMP3 BRR_TEMP4 BRR_TEMP5 MEAN_TEMP BRRDATA;
RUN;

PROC SORT DATA=BRR_TEMP_VAR OUT=BRR_TEMP1;
        BY &BYVAR L COVPARM;
RUN;

DATA BRR_TEMP2(DROP=STAT FIN1-FIN5 MESVAR)BRR_TEMP3(KEEP=&BYVAR COVPARM STAT FIN1-FIN5 MESVAR);
        SET BRR_TEMP1;
        IF L > 0 THEN OUTPUT BRR_TEMP2;
        ELSE DO;
                STAT =(PV1+PV2+PV3+PV4+PV5)/5;
                FIN1=PV1;
                FIN2=PV2;
                FIN3=PV3;
                FIN4=PV4;
                FIN5=PV5;
                MESVAR=(((STAT-FIN1)**2)+((STAT-FIN2)**2)+((STAT-FIN3)**2)+((STAT-FIN4)**2)+((STAT-FIN5)**2))/4;
                OUTPUT BRR_TEMP3;
        END;
RUN;

PROC SORT DATA=BRR_TEMP2;
        BY &BYVAR COVPARM L;
RUN;

PROC SORT DATA=BRR_TEMP3;
        BY &BYVAR COVPARM;
RUN;

DATA BRR_TEMP4;
        MERGE BRR_TEMP2 BRR_TEMP3;
        BY &BYVAR COVPARM;
        ARRAY A (5)
                PV1-PV5;
        ARRAY B (5)
                FIN1-FIN5;
        ARRAY C(5)
                VAR1-VAR5;
        DO I=1 TO 5;
                C(I)=(1/20)*((A(I)-B(I))**2);
        END;
RUN;

PROC UNIVARIATE DATA=BRR_TEMP4 NOPRINT;
        VAR VAR1 VAR2 VAR3 VAR4 VAR5;
        BY &BYVAR COVPARM;
        OUTPUT OUT=BRR_TEMP5 SUM=SS1 SS2 SS3 SS4 SS5;
RUN;

DATA &OUTFILE._VARIANCE;
        MERGE BRR_TEMP3 BRR_TEMP5;
        BY &BYVAR COVPARM;
        SAMP=(SS1+SS2+SS3+SS4+SS5)/5;
        FINVAR=(SAMP+(1.2*MESVAR));
        SESTAT=(FINVAR)**0.5;
        FORMAT STAT F10.1;
        FORMAT SESTAT F10.2;
        KEEP &BYVAR COVPARM STAT SESTAT;
RUN;

PROC DATASETS LIBRARY=WORK NOLIST;
        DELETE  BRR_TEMP1 BRR_TEMP2 BRR_TEMP3 BRR_TEMP4 BRR_TEMP5 MEAN_TEMP /*BRRDATA*/;
RUN;

PROC SORT DATA=BRR_TEMP_VAR ;
        BY &BYVAR L COVPARM;
RUN;

%DO K=1 %TO 5;

PROC TRANSPOSE DATA=BRR_TEMP_VAR OUT=BRR_TEMP0;
        VAR PV&K;
        BY &BYVAR L;
        ID COVPARM;
RUN;

DATA BRR_TEMP0;
        SET BRR_TEMP0;
        PV&K=(INTERCEPT/(INTERCEPT+RESIDUAL));
        KEEP &BYVAR L PV&K;
RUN;

%IF (&K=1) %THEN %DO;
        DATA BRR_TEMP1;
                SET BRR_TEMP0;
        RUN;
%END;
DATA BRR_TEMP1;
        MERGE BRR_TEMP1 BRR_TEMP0;
        BY &BYVAR L;
RUN;

%END;


DATA BRR_TEMP2(DROP=STAT FIN1-FIN5 MESVAR)BRR_TEMP3(KEEP=&BYVAR STAT FIN1-FIN5 MESVAR);
        SET BRR_TEMP1;
        IF L > 0 THEN OUTPUT BRR_TEMP2;
        ELSE DO;
                STAT =(PV1+PV2+PV3+PV4+PV5)/5;
                FIN1=PV1;
                FIN2=PV2;
                FIN3=PV3;
                FIN4=PV4;
                FIN5=PV5;
                MESVAR=(((STAT-FIN1)**2)+((STAT-FIN2)**2)+((STAT-FIN3)**2)+((STAT-FIN4)**2)+((STAT-FIN5)**2))/4;
                OUTPUT BRR_TEMP3;
        END;
RUN;

PROC SORT DATA=BRR_TEMP2;
        BY &BYVAR  L;
RUN;

PROC SORT DATA=BRR_TEMP3;
        BY &BYVAR ;
RUN;

DATA BRR_TEMP4;
        MERGE BRR_TEMP2 BRR_TEMP3;
        BY &BYVAR ;
        ARRAY A (5)
                PV1-PV5;
        ARRAY B (5)
                FIN1-FIN5;
        ARRAY C(5)
                VAR1-VAR5;
        DO I=1 TO 5;
                C(I)=(1/20)*((A(I)-B(I))**2);
        END;
RUN;

PROC UNIVARIATE DATA=BRR_TEMP4 NOPRINT;
        VAR VAR1 VAR2 VAR3 VAR4 VAR5;
        BY &BYVAR ;
        OUTPUT OUT=BRR_TEMP5 SUM=SS1 SS2 SS3 SS4 SS5;
RUN;

DATA &OUTFILE._INTRACLASS;
        MERGE BRR_TEMP3 BRR_TEMP5;
        BY &BYVAR ;
        SAMP=(SS1+SS2+SS3+SS4+SS5)/5;
        FINVAR=(SAMP+(1.2*MESVAR));
        SESTAT=(FINVAR)**0.5;
        FORMAT STAT F10.2;
        FORMAT SESTAT F10.2;
        KEEP &BYVAR STAT SESTAT;
RUN;

PROC DATASETS LIBRARY=WORK NOLIST;
        DELETE BRR_TEMP_VAR COV_TEMP  BRR_TEMP0 BRR_TEMP1 BRR_TEMP2 BRR_TEMP3 BRR_TEMP4 BRR_TEMP5 MEAN_TEMP
        BRR_PREP1 BRR_PREP2 BRR_PREP3;
RUN;

PROC PRINTTO PRINT=PRINT;
RUN;

OPTIONS NOTES;

%MEND;
