%MACRO QUARTILE_PV(INFILE=,
				   REPLI_ROOT=,
                   BYVAR = ,
                   PV_ROOT =,
                   INDEX =,
			   	   LIMIT=,
			   	   LIMIT_CRITERIA=,
			   	   ID_SCHOOL=,
                   OUTFILE =);


/* 

MEANING OF THE MACRO ARGUMENTS

INFILE			= 	INPUT DATA FILE.
REPLI_ROOT		= 	ROOT OF THE FINAL WEIGHT AND 80 REPLICATES VARIABLE NAMES. 
					FINAL WEIGHT VARIABLE NAME MUST BE THE REPLICATION ROOT FOLLOWED BY 0.
BYVAR			= 	BREAKDOWN VARIABLES
PV_ROOT			= 	ROOT OF THE 5 PLAUSIBLE VALUES VARIABLES NAMES
INDEX			=   VARIABLE NAME USED TO CREATE THE FOUR QUARTERS
LIMIT          	=    FLAGGING  YES OR NO
LIMIT_CRITERIA 	=   1) NUMBER OF STUDENTS 2) NUMBER OF SCHOOLS 3) PERCENTAGE OF STUDENTS AND 4) NUMBER OF VARIABLES FROM 
					THE BYVAR ARGUMENT FOR DEFINING THE POPULATION OF REFERENCE 
ID_SCHOOL	   	=	VARIABLE NAME FOR THE SCHOOL IDENTIFICATION

OUTFILE			= 	FILE WITH THE STATISTIC ESTIMATES AND THEIR STANDARD ERROR ESTIMATE

*/ 





OPTIONS NONOTES;

DATA QUARTILE_TEMP1 (KEEP=&BYVAR &REPLI_ROOT.0-&REPLI_ROOT.80 &PV_ROOT.1-&PV_ROOT.5 INDEX1-INDEX5 &ID_SCHOOL &INDEX NB_MISS) ;
	SET &INFILE;
	NB_MISS=0;
	ARRAY A (6) &INDEX &PV_ROOT.1-&PV_ROOT.5;
	DO I=1 TO 6;
		IF (A(I) IN (.,.M,.N,.I)) THEN NB_MISS=NB_MISS+1;
	END;
	INDEX1=&INDEX + (0.01*normal(-01));
	INDEX2=&INDEX + (0.01*normal(-23));
	INDEX3=&INDEX + (0.01*normal(-45));
	INDEX4=&INDEX + (0.01*normal(-67));
	INDEX5=&INDEX + (0.01*normal(-89));
RUN;

PROC SORT  DATA=QUARTILE_TEMP1;
	BY &BYVAR;
RUN;

PROC MEANS DATA=QUARTILE_TEMP1 NOPRINT;
	VAR INDEX1-INDEX5;
	BY &BYVAR;
	WEIGHT &REPLI_ROOT.0;
	WHERE (NB_MISS=0);
	OUTPUT OUT=QUARTILE_TEMP2 P25=Q1_1-Q1_5 P50=Q2_1-Q2_5 P75=Q3_1-Q3_5;
RUN;

DATA 
	QUARTILE_TEMP3 (KEEP = &BYVAR &REPLI_ROOT.0-&REPLI_ROOT.80 &PV_ROOT.1-&PV_ROOT.5 
							INDEX1-INDEX5 CAT1-CAT5 &ID_SCHOOL &INDEX NB_MISS)
	QUARTILE_TEMP4 (KEEP = &BYVAR CAT1-CAT5 &REPLI_ROOT.0 NB_MISS &ID_SCHOOL &INDEX);
	MERGE QUARTILE_TEMP1 QUARTILE_TEMP2;
	BY &BYVAR;
	ARRAY A1 (5) INDEX1-INDEX5;
	ARRAY A2 (5) Q1_1-Q1_5;
	ARRAY A3 (5) Q2_1-Q2_5;
	ARRAY A4 (5) Q3_1-Q3_5;
	ARRAY A5 (5) CAT1-CAT5;
	DO I=1 TO 5;
		IF (A1(i) <= A2(i)) THEN A5(i)=1;
		IF (A1(i) >  A2(i) AND A1(i) <= A3(i)) THEN A5(i)=2;
		IF (A1(i) >  A3(i) AND A1(i) <= A4(i)) THEN A5(i)=3;
		IF (A1(i) >  A4(i)) THEN A5(i)=4;
		IF (A1(i) IN (.,.M,.N,.I)) THEN A5(i) = .;
	END;
	IF (NB_MISS=0) THEN OUTPUT QUARTILE_TEMP3;
	OUTPUT QUARTILE_TEMP4; 
RUN;

%DO QUARTILE=1 %TO 5;

%SUBQUARTILE1(INFILE =QUARTILE_TEMP3,
              REPLI_ROOT =&REPLI_ROOT,
              SUB_BY_VAR = &BYVAR CAT&QUARTILE ,
              VAR =&INDEX,
              OUTFILE =INDEX&QUARTILE);
RUN;

DATA INDEX&QUARTILE;
	SET INDEX&QUARTILE;
	STAT&QUARTILE=STAT;
	SESTAT&QUARTILE=SESTAT;
	CAT=CAT&QUARTILE;
	KEEP &BYVAR CAT STAT&QUARTILE SESTAT&QUARTILE ;
RUN;

%END;

DATA QUARTILE_TEMP5;
	MERGE INDEX1 INDEX2 INDEX3 INDEX4 INDEX5;
	BY &BYVAR CAT;
	STAT=(STAT1+STAT2+STAT3+STAT4+STAT5)/5;
	VAR_IMP=(((STAT1-STAT)**2)+((STAT2-STAT)**2)+((STAT3-STAT)**2)+((STAT4-STAT)**2)+((STAT5-STAT)**2))/4;
	VAR_SAMP=((SESTAT1**2)+(SESTAT2**2)+(SESTAT3**2)+(SESTAT4**2)+(SESTAT5**2))/5;
	SESTAT=(VAR_SAMP+(1.2*VAR_IMP))**0.5;
	KEEP &BYVAR CAT STAT SESTAT;
RUN;

%DO QUARTILE=1 %TO 5;

%SUBQUARTILE1(INFILE =QUARTILE_TEMP3,
              REPLI_ROOT =&REPLI_ROOT,
              SUB_BY_VAR = &BYVAR CAT&QUARTILE ,
              VAR =&PV_ROOT&QUARTILE,
              OUTFILE =RESULT&QUARTILE);
RUN;

DATA RESULT&QUARTILE;
	SET RESULT&QUARTILE;
	STAT&QUARTILE=STAT;
	SESTAT&QUARTILE=SESTAT;
	CAT=CAT&QUARTILE;
	KEEP &BYVAR CAT STAT&QUARTILE SESTAT&QUARTILE ;
RUN;

%END;

DATA QUARTILE_TEMP6;
	MERGE RESULT1 RESULT2 RESULT3 RESULT4 RESULT5;
	BY &BYVAR CAT;
	STAT=(STAT1+STAT2+STAT3+STAT4+STAT5)/5;
	VAR_IMP=(((STAT1-STAT)**2)+((STAT2-STAT)**2)+((STAT3-STAT)**2)+((STAT4-STAT)**2)+((STAT5-STAT)**2))/4;
	VAR_SAMP=((SESTAT1**2)+(SESTAT2**2)+(SESTAT3**2)+(SESTAT4**2)+(SESTAT5**2))/5;
	SESTAT=(VAR_SAMP+(1.2*VAR_IMP))**0.5;
	KEEP &BYVAR CAT STAT SESTAT;
RUN;

DATA QUARTILE_TEMP7;
	MERGE 	QUARTILE_TEMP5 (RENAME=(STAT=INDEX_STAT SESTAT=INDEX_SESTAT))
			QUARTILE_TEMP6 (RENAME=(STAT=PV_STAT SESTAT=PV_SESTAT))	;
	BY &BYVAR CAT;
RUN;


%IF (%UPCASE(&LIMIT)=NO) %THEN %DO;

	DATA &OUTFILE;
		SET QUARTILE_TEMP7;;
	RUN;

%END;
%ELSE %DO;
	PROC FREQ DATA=QUARTILE_TEMP3 NOPRINT;
		TABLE &BYVAR * CAT1 /OUT=QUARTILE_TEMP8_1;
		TABLE &BYVAR * CAT2 /OUT=QUARTILE_TEMP8_2;
		TABLE &BYVAR * CAT3 /OUT=QUARTILE_TEMP8_3;
		TABLE &BYVAR * CAT4 /OUT=QUARTILE_TEMP8_4;
		TABLE &BYVAR * CAT5 /OUT=QUARTILE_TEMP8_5;
	RUN;
	%LET FLAG_STUD=%SCAN(&LIMIT_CRITERIA,1);
	DATA QUARTILE_TEMP8;
		MERGE 	
			QUARTILE_TEMP8_1 (RENAME=(COUNT=C1 CAT1=CAT))
			QUARTILE_TEMP8_2 (RENAME=(COUNT=C2 CAT2=CAT))
			QUARTILE_TEMP8_3 (RENAME=(COUNT=C3 CAT3=CAT))
			QUARTILE_TEMP8_4 (RENAME=(COUNT=C4 CAT4=CAT))
			QUARTILE_TEMP8_5 (RENAME=(COUNT=C5 CAT5=CAT));
		BY &BYVAR ;
		C=(C1+C2+C3+C4+C5)/5;
		IF (C < &FLAG_STUD) THEN FLAG_STUD=1;
		ELSE FLAG_STUD=0;
		KEEP &BYVAR FLAG_STUD;
	RUN;

	%DO M=1 %TO 5;
	PROC FREQ DATA=QUARTILE_TEMP3 NOPRINT;
		TABLE &BYVAR * CAT&M * &ID_SCHOOL /OUT=QUARTILE_TEMP9_&M;
	RUN;		
	PROC FREQ DATA=QUARTILE_TEMP9_&M NOPRINT;
		TABLE &BYVAR * CAT&M /OUT=QUARTILE_TEMP10_&M;
	RUN;
	%END;
	%LET FLAG_SCH=%SCAN(&LIMIT_CRITERIA,2);
	DATA QUARTILE_TEMP10;
		MERGE 	
			QUARTILE_TEMP10_1 (RENAME=(COUNT=C1 CAT1=CAT))
			QUARTILE_TEMP10_2 (RENAME=(COUNT=C2 CAT2=CAT))
			QUARTILE_TEMP10_3 (RENAME=(COUNT=C3 CAT3=CAT))
			QUARTILE_TEMP10_4 (RENAME=(COUNT=C4 CAT4=CAT))
			QUARTILE_TEMP10_5 (RENAME=(COUNT=C5 CAT5=CAT));
		BY &BYVAR CAT;
		C=(C1+C2+C3+C4+C5)/5;
		IF (C < &FLAG_SCH) THEN FLAG_SCH=1;
		ELSE FLAG_SCH=0;
		KEEP &BYVAR FLAG_SCH;
	RUN;

 	%LET K=1;
	%LET POPREF=;
	%LET NBVAR=%SCAN(&LIMIT_CRITERIA,4);
	%DO %WHILE(&K <= &NBVAR);
		%LET POPREF_ADD=%SCAN(&BYVAR,&K);
		%LET POPREF=&POPREF &POPREF_ADD;
		%LET K=%EVAL(&K+1);
	%END;

	%LET FLAG_PCT=%SCAN(&LIMIT_CRITERIA,3);

	PROC FREQ DATA=QUARTILE_TEMP4 NOPRINT;
		TABLE &BYVAR * CAT1 * NB_MISS / OUT=QUARTILE_TEMP11_1;
		TABLE &BYVAR * CAT2 * NB_MISS / OUT=QUARTILE_TEMP11_2;
		TABLE &BYVAR * CAT3 * NB_MISS / OUT=QUARTILE_TEMP11_3;
		TABLE &BYVAR * CAT4 * NB_MISS / OUT=QUARTILE_TEMP11_4;
		TABLE &BYVAR * CAT5 * NB_MISS / OUT=QUARTILE_TEMP11_5;
		WEIGHT W_FSTR0;
	RUN;

	DATA QUARTILE_TEMP11;
		MERGE 	
			QUARTILE_TEMP11_1 (RENAME=(COUNT=C1 CAT1=CAT))
			QUARTILE_TEMP11_2 (RENAME=(COUNT=C2 CAT2=CAT))
			QUARTILE_TEMP11_3 (RENAME=(COUNT=C3 CAT3=CAT))
			QUARTILE_TEMP11_4 (RENAME=(COUNT=C4 CAT4=CAT))
			QUARTILE_TEMP11_5 (RENAME=(COUNT=C5 CAT5=CAT));
		BY &BYVAR CAT NB_MISS;
	RUN;	

	PROC MEANS DATA=QUARTILE_TEMP11 NOPRINT;
		VAR C1-C5;
		BY &BYVAR;
  		OUTPUT OUT=QUARTILE_TEMP12 SUM=SOMWGT1-SOMWGT5;
	RUN;

	DATA QUARTILE_TEMP13;
		MERGE QUARTILE_TEMP11 QUARTILE_TEMP12;
		BY &BYVAR;
		IF (NB_MISS=0);
		ARRAY A1 (5) C1-C5;
		ARRAY A2 (5) SOMWGT1-SOMWGT5;
		ARRAY A3 (5) PCT1-PCT5;
		DO I=1 TO 5;
			A3(I)=(A1(I)/A2(I))*100;
		END;
		PCT=(PCT1+PCT2+PCT3+PCT4+PCT5)/5;
		IF (PCT < &FLAG_PCT) THEN FLAG_PCT=1;
		ELSE FLAG_PCT=0;
		KEEP &BYVAR FLAG_PCT;
	RUN;
	DATA &OUTFILE;
		MERGE QUARTILE_TEMP7 QUARTILE_TEMP8 QUARTILE_TEMP10 QUARTILE_TEMP13;
		BY &BYVAR;
	RUN;
	
	PROC DATASETS LIBRARY=WORK NOLIST;
		DELETE 
			QUARTILE_TEMP8_1 QUARTILE_TEMP8_2 QUARTILE_TEMP8_3 QUARTILE_TEMP8_4 QUARTILE_TEMP8_5
			QUARTILE_TEMP8 
			QUARTILE_TEMP9_1 QUARTILE_TEMP9_2 QUARTILE_TEMP9_3 QUARTILE_TEMP9_4 QUARTILE_TEMP9_5
			QUARTILE_TEMP10_1 QUARTILE_TEMP10_2 QUARTILE_TEMP10_3 QUARTILE_TEMP10_4 QUARTILE_TEMP10_5
			QUARTILE_TEMP10
			QUARTILE_TEMP11_1 QUARTILE_TEMP11_2 QUARTILE_TEMP11_3 QUARTILE_TEMP11_4 QUARTILE_TEMP11_5
			QUARTILE_TEMP11 QUARTILE_TEMP12 QUARTILE_TEMP13;
	RUN;

%END;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE  QUARTILE_TEMP1 QUARTILE_TEMP2 QUARTILE_TEMP3 QUARTILE_TEMP4 QUARTILE_TEMP5 QUARTILE_TEMP6 QUARTILE_TEMP7
			INDEX1 INDEX2 INDEX3 INDEX4 INDEX5
			RESULT1 RESULT2 RESULT3 RESULT4 RESULT5;
RUN;

OPTION NOTES;

%MEND ;


%MACRO SUBQUARTILE1(INFILE =,
                    REPLI_ROOT =,
                    SUB_BY_VAR = ,
                    VAR =,
                    OUTFILE =);

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP1;
RUN;

PROC SORT DATA=&INFILE OUT=BRRDATA(KEEP=&REPLI_ROOT.0-&REPLI_ROOT.80 &SUB_BY_VAR &VAR);
  BY &SUB_BY_VAR;
RUN;

%DO I = 0 %TO 80;
 	PROC MEANS DATA=BRRDATA VARDEF=WGT NOPRINT;
		VAR &VAR ;
		BY &SUB_BY_VAR;
		WEIGHT &REPLI_ROOT&I;
		OUTPUT OUT=MEAN_TEMP MEAN=PV;
	RUN;

	DATA MEAN_TEMP;
		SET MEAN_TEMP;
		L=&I;
	RUN;

	PROC APPEND BASE = BRR_TEMP1 DATA=MEAN_TEMP;
 	RUN;
%END;

DATA BRR_TEMP2(KEEP=&SUB_BY_VAR PV)BRR_TEMP3(KEEP=&SUB_BY_VAR STAT);
	SET BRR_TEMP1;
	IF L > 0 THEN OUTPUT BRR_TEMP2;     
	ELSE DO;
		STAT =PV;
		OUTPUT BRR_TEMP3;
	END;
RUN;

PROC SORT DATA=BRR_TEMP2;
	BY &SUB_BY_VAR;
RUN;

PROC SORT DATA=BRR_TEMP3;
	BY &SUB_BY_VAR;
RUN;

DATA BRR_TEMP4;
	MERGE BRR_TEMP2 BRR_TEMP3;
	BY &SUB_BY_VAR;
	VARI=((PV-STAT)**2)*(1/20);
RUN;

PROC UNIVARIATE DATA=BRR_TEMP4 NOPRINT;
	VAR VARI;
	BY &SUB_BY_VAR;
	OUTPUT OUT=BRR_TEMP5 SUM=SS;
RUN;


DATA &OUTFILE;
	MERGE BRR_TEMP3 BRR_TEMP5;
	BY &SUB_BY_VAR;
	SESTAT=(SS)**0.5;
	FORMAT STAT F10.2;
	FORMAT SESTAT F10.2;
	KEEP &SUB_BY_VAR STAT SESTAT;
RUN;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP1 BRR_TEMP2 BRR_TEMP3 BRR_TEMP4 BRR_TEMP5 BRR_TEMP6 MEAN_TEMP BRRDATA;
RUN;

%MEND SUBQUARTILE1;

